@@ -49,6 +49,7 @@
 #include "OpenImageIO/strutil.h"
 #include "OpenImageIO/fmath.h"
 #include "OpenImageIO/thread.h"
+#include "OpenImageIO/simd.h"
 
 OIIO_NAMESPACE_ENTER
 {
@@ -133,19 +134,19 @@ roi_intersection (const ROI &A, const ROI &B)
 // detail.
 class ImageBufImpl {
 public:
-    ImageBufImpl (const std::string &filename, int subimage, int miplevel,
+    ImageBufImpl (string_view filename, int subimage, int miplevel,
                   ImageCache *imagecache=NULL, const ImageSpec *spec=NULL,
-                  void *buffer=NULL);
+                  void *buffer=NULL, const ImageSpec *config = NULL);
     ImageBufImpl (const ImageBufImpl &src);
     ~ImageBufImpl ();
 
     void clear ();
-    void reset (const std::string &name, int subimage, int miplevel,
-                ImageCache *imagecache);
-    void reset (const std::string &name, const ImageSpec &spec);
+    void reset (string_view name, int subimage, int miplevel,
+                ImageCache *imagecache, const ImageSpec *config);
+    void reset (string_view name, const ImageSpec &spec);
     void alloc (const ImageSpec &spec);
     void realloc ();
-    bool init_spec (const std::string &filename, int subimage, int miplevel);
+    bool init_spec (string_view filename, int subimage, int miplevel);
     bool read (int subimage=0, int miplevel=0, bool force=false,
                TypeDesc convert=TypeDesc::UNKNOWN,
                ProgressCallback progress_callback=NULL,
@@ -256,7 +257,6 @@ private:
     mutable bool m_spec_valid;   ///< Is the spec valid
     mutable bool m_pixels_valid; ///< Image is valid
     bool m_badfile;              ///< File not found
-    int m_orientation;           ///< Orientation of the image
     float m_pixelaspect;         ///< Pixel aspect ratio of the image
     size_t m_pixel_bytes;
     size_t m_scanline_bytes;
@@ -270,6 +270,7 @@ private:
     int m_write_tile_width;
     int m_write_tile_height;
     int m_write_tile_depth;
+    boost::scoped_ptr<ImageSpec> m_configspec; // Configuration spec
     mutable std::string m_err;   ///< Last error message
 
     const ImageBufImpl operator= (const ImageBufImpl &src); // unimplemented
@@ -278,16 +279,17 @@ private:
 
 
 
-ImageBufImpl::ImageBufImpl (const std::string &filename,
+ImageBufImpl::ImageBufImpl (string_view filename,
                             int subimage, int miplevel,
                             ImageCache *imagecache,
-                            const ImageSpec *spec, void *buffer)
+                            const ImageSpec *spec, void *buffer,
+                            const ImageSpec *config)
     : m_storage(ImageBuf::UNINITIALIZED),
       m_name(filename), m_nsubimages(0),
       m_current_subimage(subimage), m_current_miplevel(miplevel),
       m_localpixels(NULL),
       m_spec_valid(false), m_pixels_valid(false),
-      m_badfile(false), m_orientation(1), m_pixelaspect(1), 
+      m_badfile(false), m_pixelaspect(1),
       m_pixel_bytes(0), m_scanline_bytes(0), m_plane_bytes(0),
       m_imagecache(imagecache), m_allocated_size(0),
       m_write_format(TypeDesc::UNKNOWN), m_write_tile_width(0),
@@ -299,7 +301,8 @@ ImageBufImpl::ImageBufImpl (const std::string &filename,
         m_pixel_bytes = spec->pixel_bytes();
         m_scanline_bytes = spec->scanline_bytes();
         m_plane_bytes = clamped_mult64 (m_scanline_bytes, (imagesize_t)m_spec.height);
-        m_blackpixel.resize (m_pixel_bytes, 0);
+        m_blackpixel.resize (round_to_multiple (m_pixel_bytes, OIIO_SIMD_MAX_SIZE_BYTES), 0);
+        // NB make it big enough for SSE
         if (buffer) {
             m_localpixels = (char *)buffer;
             m_storage = ImageBuf::APPBUFFER;
@@ -313,6 +316,7 @@ ImageBufImpl::ImageBufImpl (const std::string &filename,
         // If a filename was given, read the spec and set it up as an
         // ImageCache-backed image.  Reallocate later if an explicit read()
         // is called to force read into a local buffer.
+        m_configspec.reset (config ? new ImageSpec (*config) : NULL);
         read (subimage, miplevel);
     } else {
         ASSERT (buffer == NULL);
@@ -332,7 +336,6 @@ ImageBufImpl::ImageBufImpl (const ImageBufImpl &src)
       m_pixels(src.m_localpixels ? new char [src.m_spec.image_bytes()] : NULL),
       m_localpixels(m_pixels.get()),
       m_badfile(src.m_badfile),
-      m_orientation(src.m_orientation),
       m_pixelaspect(src.m_pixelaspect),
       m_pixel_bytes(src.m_pixel_bytes),
       m_scanline_bytes(src.m_scanline_bytes),
@@ -363,6 +366,8 @@ ImageBufImpl::ImageBufImpl (const ImageBufImpl &src)
         // Source was cache-based or deep
         // nothing else to do
     }
+    if (src.m_configspec)
+        m_configspec.reset (new ImageSpec(*src.m_configspec));
 }
 
 
@@ -385,15 +390,16 @@ ImageBuf::ImageBuf ()
 
 
 
-ImageBuf::ImageBuf (const std::string &filename, int subimage, int miplevel,
-                    ImageCache *imagecache)
-    : m_impl (new ImageBufImpl (filename, subimage, miplevel, imagecache))
+ImageBuf::ImageBuf (string_view filename, int subimage, int miplevel,
+                    ImageCache *imagecache, const ImageSpec *config)
+    : m_impl (new ImageBufImpl (filename, subimage, miplevel, imagecache,
+              NULL /*spec*/, NULL /*buffer*/, config))
 {
 }
 
 
 
-ImageBuf::ImageBuf (const std::string &filename, ImageCache *imagecache)
+ImageBuf::ImageBuf (string_view filename, ImageCache *imagecache)
     : m_impl (new ImageBufImpl (filename, 0, 0, imagecache))
 {
 }
@@ -401,22 +407,22 @@ ImageBuf::ImageBuf (const std::string &filename, ImageCache *imagecache)
 
 
 ImageBuf::ImageBuf (const ImageSpec &spec)
-    : m_impl (new ImageBufImpl (std::string(), 0, 0, NULL, &spec))
+    : m_impl (new ImageBufImpl ("", 0, 0, NULL, &spec))
 {
-    alloc (spec);
+    m_impl->alloc (spec);
 }
 
 
 
-ImageBuf::ImageBuf (const std::string &filename, const ImageSpec &spec)
+ImageBuf::ImageBuf (string_view filename, const ImageSpec &spec)
     : m_impl (new ImageBufImpl (filename, 0, 0, NULL, &spec))
 {
-    alloc (spec);
+    m_impl->alloc (spec);
 }
 
 
 
-ImageBuf::ImageBuf (const std::string &filename, const ImageSpec &spec,
+ImageBuf::ImageBuf (string_view filename, const ImageSpec &spec,
                     void *buffer)
     : m_impl (new ImageBufImpl (filename, 0, 0, NULL, &spec, buffer))
 {
@@ -425,7 +431,7 @@ ImageBuf::ImageBuf (const std::string &filename, const ImageSpec &spec,
 
 
 ImageBuf::ImageBuf (const ImageSpec &spec, void *buffer)
-    : m_impl (new ImageBufImpl (std::string(), 0, 0, NULL, &spec, buffer))
+    : m_impl (new ImageBufImpl ("", 0, 0, NULL, &spec, buffer))
 {
 }
 
@@ -513,7 +519,6 @@ ImageBufImpl::clear ()
     m_spec_valid = false;
     m_pixels_valid = false;
     m_badfile = false;
-    m_orientation = 1;
     m_pixelaspect = 1;
     m_pixel_bytes = 0;
     m_scanline_bytes = 0;
@@ -525,6 +530,7 @@ ImageBufImpl::clear ()
     m_write_tile_width = 0;
     m_write_tile_height = 0;
     m_write_tile_depth = 0;
+    m_configspec.reset (NULL);
 }
 
 
@@ -538,8 +544,9 @@ ImageBuf::clear ()
 
 
 void
-ImageBufImpl::reset (const std::string &filename, int subimage,
-                     int miplevel, ImageCache *imagecache)
+ImageBufImpl::reset (string_view filename, int subimage,
+                     int miplevel, ImageCache *imagecache,
+                     const ImageSpec *config)
 {
     clear ();
     m_name = ustring (filename);
@@ -547,6 +554,8 @@ ImageBufImpl::reset (const std::string &filename, int subimage,
     m_current_miplevel = miplevel;
     if (imagecache)
         m_imagecache = imagecache;
+    if (config)
+        m_configspec.reset (new ImageSpec(*config));
 
     if (m_name.length() > 0) {
         // If a filename was given, read the spec and set it up as an
@@ -559,24 +568,24 @@ ImageBufImpl::reset (const std::string &filename, int subimage,
 
 
 void
-ImageBuf::reset (const std::string &filename, int subimage, int miplevel,
-                 ImageCache *imagecache)
+ImageBuf::reset (string_view filename, int subimage, int miplevel,
+                 ImageCache *imagecache, const ImageSpec *config)
 {
-    impl()->reset (filename, subimage, miplevel, imagecache);
+    impl()->reset (filename, subimage, miplevel, imagecache, config);
 }
 
 
 
 void
-ImageBuf::reset (const std::string &filename, ImageCache *imagecache)
+ImageBuf::reset (string_view filename, ImageCache *imagecache)
 {
-    impl()->reset (filename, 0, 0, imagecache);
+    impl()->reset (filename, 0, 0, imagecache, NULL);
 }
 
 
 
 void
-ImageBufImpl::reset (const std::string &filename, const ImageSpec &spec)
+ImageBufImpl::reset (string_view filename, const ImageSpec &spec)
 {
     clear ();
     m_name = ustring (filename);
@@ -588,7 +597,7 @@ ImageBufImpl::reset (const std::string &filename, const ImageSpec &spec)
 
 
 void
-ImageBuf::reset (const std::string &filename, const ImageSpec &spec)
+ImageBuf::reset (string_view filename, const ImageSpec &spec)
 {
     impl()->reset (filename, spec);
 }
@@ -598,7 +607,7 @@ ImageBuf::reset (const std::string &filename, const ImageSpec &spec)
 void
 ImageBuf::reset (const ImageSpec &spec)
 {
-    impl()->reset (std::string(), spec);
+    impl()->reset ("", spec);
 }
 
 
@@ -615,9 +624,18 @@ ImageBufImpl::realloc ()
     m_pixel_bytes = m_spec.pixel_bytes();
     m_scanline_bytes = m_spec.scanline_bytes();
     m_plane_bytes = clamped_mult64 (m_scanline_bytes, (imagesize_t)m_spec.height);
-    m_blackpixel.resize (m_pixel_bytes, 0);
+    m_blackpixel.resize (round_to_multiple (m_pixel_bytes, OIIO_SIMD_MAX_SIZE_BYTES), 0);
+    // NB make it big enough for SSE
     if (m_allocated_size)
         m_pixels_valid = true;
+    if (m_spec.deep) {
+        int npixels = (int)m_spec.image_pixels();
+        std::vector<TypeDesc> chanformats (m_spec.channelformats);
+        chanformats.resize (m_spec.nchannels, m_spec.format);
+        m_deepdata.init (npixels, m_spec.nchannels,
+                         &(*chanformats.begin()), &(*chanformats.end()));
+        m_storage = ImageBuf::LOCALBUFFER;
+    }
 #if 0
     std::cerr << "ImageBuf " << m_name << " local allocation: " << m_allocated_size << "\n";
 #endif
@@ -643,14 +661,6 @@ ImageBufImpl::alloc (const ImageSpec &spec)
 
 
 
-void
-ImageBuf::alloc (const ImageSpec &spec)
-{
-    impl()->alloc (spec);
-}
-
-
-
 void
 ImageBuf::copy_from (const ImageBuf &src)
 {
@@ -677,7 +687,7 @@ ImageBuf::copy_from (const ImageBuf &src)
 
 
 bool
-ImageBufImpl::init_spec (const std::string &filename, int subimage, int miplevel)
+ImageBufImpl::init_spec (string_view filename, int subimage, int miplevel)
 {
     if (!m_badfile && m_spec_valid
             && m_current_subimage >= 0 && m_current_miplevel >= 0
@@ -695,6 +705,8 @@ ImageBufImpl::init_spec (const std::string &filename, int subimage, int miplevel
     m_nmiplevels = 0;
     static ustring s_subimages("subimages"), s_miplevels("miplevels");
     static ustring s_fileformat("fileformat");
+    if (m_configspec)  // Pass configuration options to cache
+        m_imagecache->add_file (m_name, NULL, m_configspec.get());
     m_imagecache->get_image_info (m_name, subimage, miplevel, s_subimages,
                                   TypeDesc::TypeInt, &m_nsubimages);
     m_imagecache->get_image_info (m_name, subimage, miplevel, s_miplevels,
@@ -708,7 +720,8 @@ ImageBufImpl::init_spec (const std::string &filename, int subimage, int miplevel
     m_pixel_bytes = m_spec.pixel_bytes();
     m_scanline_bytes = m_spec.scanline_bytes();
     m_plane_bytes = clamped_mult64 (m_scanline_bytes, (imagesize_t)m_spec.height);
-    m_blackpixel.resize (m_pixel_bytes, 0);
+    m_blackpixel.resize (round_to_multiple (m_pixel_bytes, OIIO_SIMD_MAX_SIZE_BYTES), 0);
+    // NB make it big enough for SSE
 
     // Subtlety: m_nativespec will have the true formats of the file, but
     // we rig m_spec to reflect what it will look like in the cache.
@@ -727,7 +740,6 @@ ImageBufImpl::init_spec (const std::string &filename, int subimage, int miplevel
 
     if (m_nsubimages) {
         m_badfile = false;
-        m_orientation = m_spec.get_int_attribute ("orientation", 1);
         m_pixelaspect = m_spec.get_float_attribute ("pixelaspectratio", 1.0f);
         m_current_subimage = subimage;
         m_current_miplevel = miplevel;
@@ -747,7 +759,7 @@ ImageBufImpl::init_spec (const std::string &filename, int subimage, int miplevel
 
 
 bool
-ImageBuf::init_spec (const std::string &filename, int subimage, int miplevel)
+ImageBuf::init_spec (string_view filename, int subimage, int miplevel)
 {
     return impl()->init_spec (filename, subimage, miplevel);
 }
@@ -776,7 +788,8 @@ ImageBufImpl::read (int subimage, int miplevel, bool force, TypeDesc convert,
     m_current_miplevel = miplevel;
 
     if (m_spec.deep) {
-        boost::scoped_ptr<ImageInput> input (ImageInput::open (m_name.string()));
+        boost::scoped_ptr<ImageInput> input (
+                    ImageInput::open (m_name.string(), m_configspec.get()));
         if (! input) {
             error ("%s", OIIO::geterror());
             return false;
@@ -810,7 +823,8 @@ ImageBufImpl::read (int subimage, int miplevel, bool force, TypeDesc convert,
         m_pixel_bytes = m_spec.pixel_bytes();
         m_scanline_bytes = m_spec.scanline_bytes();
         m_plane_bytes = clamped_mult64 (m_scanline_bytes, (imagesize_t)m_spec.height);
-        m_blackpixel.resize (m_pixel_bytes, 0);
+        m_blackpixel.resize (round_to_multiple (m_pixel_bytes, OIIO_SIMD_MAX_SIZE_BYTES), 0);
+        // NB make it big enough for SSE
         m_pixels_valid = true;
         m_storage = ImageBuf::IMAGECACHE;
 #ifndef NDEBUG
@@ -833,21 +847,56 @@ ImageBufImpl::read (int subimage, int miplevel, bool force, TypeDesc convert,
         m_spec.format = convert;
     else
         m_spec.format = m_nativespec.format;
-    m_orientation = m_spec.get_int_attribute ("orientation", 1);
     m_pixelaspect = m_spec.get_float_attribute ("pixelaspectratio", 1.0f);
     realloc ();
+
+    // If forcing a full read, make sure the spec reflects the nativespec's
+    // tile sizes, rather than that imposed by the ImageCache.
+    m_spec.tile_width = m_nativespec.tile_width;
+    m_spec.tile_height = m_nativespec.tile_height;
+    m_spec.tile_depth = m_nativespec.tile_depth;
+
+    if (convert != TypeDesc::UNKNOWN &&
+        convert != m_cachedpixeltype &&
+        convert.size() >= m_cachedpixeltype.size() &&
+        convert.size() >= m_nativespec.format.size()) {
+        // A specific conversion type was requested which is not the cached
+        // type and whose bit depth is as much or more than the cached type.
+        // Bypass the cache and read directly so that there is no possible
+        // loss of range or precision resulting from going through the
+        // cache.
+        ImageInput *in = ImageInput::open (m_name.string(), m_configspec.get());
+        bool ok = true;
+        if (in) {
+            if (subimage || miplevel) {
+                ImageSpec newspec;
+                ok &= in->seek_subimage (subimage, miplevel, newspec);
+            }
+            if (ok)
+                ok &= in->read_image (convert, m_localpixels);
+            in->close ();
+            if (ok) {
+                m_pixels_valid = true;
+            } else {
+                m_pixels_valid = false;
+                error ("%s", in->geterror());
+            }
+            delete in;
+        } else {
+            m_pixels_valid = false;
+            error ("%s", OIIO::geterror());
+        }
+        return m_pixels_valid;
+    }
+
+    // All other cases, no loss of precision is expected, so even a forced
+    // read should go through the image cache.
     if (m_imagecache->get_pixels (m_name, subimage, miplevel,
                                   m_spec.x, m_spec.x+m_spec.width,
                                   m_spec.y, m_spec.y+m_spec.height,
                                   m_spec.z, m_spec.z+m_spec.depth,
                                   m_spec.format, m_localpixels)) {
         m_pixels_valid = true;
-        // If forcing a full read, make sure the spec reflects the
-        // nativespec's tile sizes, rather than that imposed by the
-        // ImageCache.
-        m_spec.tile_width = m_nativespec.tile_width;
-        m_spec.tile_height = m_nativespec.tile_height;
-        m_spec.tile_depth = m_nativespec.tile_depth;
     } else {
         m_pixels_valid = false;
         error ("%s", m_imagecache->geterror ());
@@ -894,7 +943,7 @@ ImageBuf::write (ImageOutput *out,
     stride_t as = AutoStride;
     bool ok = true;
     const ImageBufImpl *impl = this->impl();
-    impl->validate_pixels ();
+    ok &= impl->validate_pixels ();
     const ImageSpec &m_spec (impl->m_spec);
     if (impl->m_localpixels) {
         // In-core pixel buffer for the whole image
@@ -906,10 +955,10 @@ ImageBuf::write (ImageOutput *out,
     } else {
         // Backed by ImageCache
         boost::scoped_array<char> tmp (new char [m_spec.image_bytes()]);
-        get_pixels (xbegin(), xend(), ybegin(), yend(), zbegin(), zend(),
-                    m_spec.format, &tmp[0]);
-        ok = out->write_image (m_spec.format, &tmp[0], as, as, as,
-                               progress_callback, progress_callback_data);
+        ok = get_pixels (xbegin(), xend(), ybegin(), yend(), zbegin(), zend(),
+                         m_spec.format, &tmp[0]);
+        ok &= out->write_image (m_spec.format, &tmp[0], as, as, as,
+                                progress_callback, progress_callback_data);
         // FIXME -- not good for huge images.  Instead, we should read
         // little bits at a time (scanline or tile blocks).
     }
@@ -921,23 +970,12 @@ ImageBuf::write (ImageOutput *out,
 
 
 bool
-ImageBuf::save (const std::string &filename, const std::string &fileformat,
-                ProgressCallback progress_callback,
-                void *progress_callback_data) const
-{
-    return write (filename, fileformat,
-                  progress_callback, progress_callback_data);
-}
-
-
-
-bool
-ImageBuf::write (const std::string &_filename, const std::string &_fileformat,
+ImageBuf::write (string_view _filename, string_view _fileformat,
                  ProgressCallback progress_callback,
                  void *progress_callback_data) const
 {
-    std::string filename = _filename.size() ? _filename : name();
-    std::string fileformat = _fileformat.size() ? _fileformat : filename;
+    string_view filename = _filename.size() ? _filename : name();
+    string_view fileformat = _fileformat.size() ? _fileformat : filename;
     if (filename.size() == 0) {
         error ("ImageBuf::write() called with no filename");
         return false;
@@ -987,6 +1025,8 @@ ImageBuf::write (const std::string &_filename, const std::string &_fileformat,
 void
 ImageBufImpl::copy_metadata (const ImageBufImpl &src)
 {
+    if (this == &src)
+        return;
     const ImageSpec &srcspec (src.spec());
     ImageSpec &m_spec (this->specmod());
     m_spec.full_x = srcspec.full_x;
@@ -1044,18 +1084,18 @@ ImageBuf::nativespec () const
 
 
 
-const std::string &
+string_view
 ImageBuf::name (void) const
 {
-    return impl()->m_name.string();
+    return impl()->m_name;
 }
 
 
-const std::string &
+string_view
 ImageBuf::file_format_name (void) const
 {
     impl()->validate_spec ();
-    return impl()->m_fileformat.string();
+    return impl()->m_fileformat;
 }
 
 
@@ -1101,7 +1141,15 @@ int
 ImageBuf::orientation () const
 {
     impl()->validate_spec();
-    return impl()->m_orientation;
+    return impl()->spec().get_int_attribute ("Orientation", 1);
+}
+
+
+
+void
+ImageBuf::set_orientation (int orient)
+{
+    impl()->specmod().attribute ("Orientation", orient);
 }
 
 
@@ -1221,6 +1269,9 @@ bool copy_pixels_2 (ImageBuf &dst, const ImageBuf &src, const ROI &roi)
 bool
 ImageBuf::copy_pixels (const ImageBuf &src)
 {
+    if (this == &src)
+        return true;
+
     // compute overlap
     ROI myroi = get_roi(spec());
     ROI roi = roi_intersection (myroi, get_roi(src.spec()));
@@ -1229,9 +1280,10 @@ ImageBuf::copy_pixels (const ImageBuf &src)
     if (roi != myroi)
         ImageBufAlgo::zero (*this);
 
-    OIIO_DISPATCH_TYPES2 ("copy_pixels", copy_pixels_2,
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "copy_pixels", copy_pixels_2,
                           spec().format, src.spec().format, *this, src, roi);
-    return true;
+    return ok;
 }
 
 
@@ -1267,8 +1319,10 @@ ImageBuf::getchannel (int x, int y, int z, int c, WrapMode wrap) const
 {
     if (c < 0 || c >= spec().nchannels)
         return 0.0f;
-    OIIO_DISPATCH_TYPES ("getchannel", getchannel_, spec().format,
+    float ret;
+    OIIO_DISPATCH_TYPES (ret, "getchannel", getchannel_, spec().format,
                          *this, x, y, z, c, wrap);
+    return ret;
 }
 
 
@@ -1290,8 +1344,10 @@ inline bool
 getpixel_wrapper (int x, int y, int z, float *pixel, int nchans,
                   ImageBuf::WrapMode wrap, const ImageBuf &ib)
 {
-    OIIO_DISPATCH_TYPES ("getpixel", getpixel_, ib.spec().format,
+    bool ok;
+    OIIO_DISPATCH_TYPES (ok, "getpixel", getpixel_, ib.spec().format,
                          ib, x, y, z, pixel, nchans, wrap);
+    return ok;
 }
 
 
@@ -1335,8 +1391,10 @@ inline bool
 interppixel_wrapper (float x, float y, float *pixel,
                      ImageBuf::WrapMode wrap, const ImageBuf &img)
 {
-    OIIO_DISPATCH_TYPES ("interppixel", interppixel_, img.spec().format,
+    bool ok;
+    OIIO_DISPATCH_TYPES (ok, "interppixel", interppixel_, img.spec().format,
                          img, x, y, pixel, wrap);
+    return ok;
 }
 
 
@@ -1353,8 +1411,8 @@ void
 ImageBuf::interppixel_NDC (float x, float y, float *pixel, WrapMode wrap) const
 {
     const ImageSpec &spec (impl()->spec());
-    interppixel (static_cast<float>(spec.x) + x * static_cast<float>(spec.width),
-                 static_cast<float>(spec.y) + y * static_cast<float>(spec.height),
+    interppixel (static_cast<float>(spec.full_x) + x * static_cast<float>(spec.full_width),
+                 static_cast<float>(spec.full_y) + y * static_cast<float>(spec.full_height),
                  pixel, wrap);
 }
 
@@ -1371,6 +1429,68 @@ ImageBuf::interppixel_NDC_full (float x, float y, float *pixel, WrapMode wrap) c
 
 
 
+template <class T>
+static bool
+interppixel_bicubic_ (const ImageBuf &img, float x, float y, float *pixel,
+                      ImageBuf::WrapMode wrap)
+{
+    int n = img.spec().nchannels;
+    x -= 0.5f;
+    y -= 0.5f;
+    int xtexel, ytexel;
+    float xfrac, yfrac;
+    xfrac = floorfrac (x, &xtexel);
+    yfrac = floorfrac (y, &ytexel);
+
+    float wx[4]; evalBSplineWeights (wx, xfrac);
+    float wy[4]; evalBSplineWeights (wy, yfrac);
+    for (int c = 0;  c < n;  ++c)
+        pixel[c] = 0.0f;
+    ImageBuf::ConstIterator<T> it (img, xtexel-1, xtexel+3, ytexel-1, ytexel+3,
+                                   0, 1, wrap);
+    for (int j = 0;  j < 4;  ++j) {
+        for (int i = 0;  i < 4;  ++i, ++it) {
+            float w = wx[i] * wy[j];
+            for (int c = 0;  c < n;  ++c)
+                pixel[c] += w * it[c];
+        }
+    }
+    return true;
+}
+
+
+
+inline bool
+interppixel_bicubic_wrapper (float x, float y, float *pixel,
+                     ImageBuf::WrapMode wrap, const ImageBuf &img)
+{
+    bool ok;
+    OIIO_DISPATCH_TYPES (ok, "interppixel_bicubic", interppixel_bicubic_,
+                         img.spec().format, img, x, y, pixel, wrap);
+    return ok;
+}
+
+
+
+void
+ImageBuf::interppixel_bicubic (float x, float y, float *pixel, WrapMode wrap) const
+{
+    interppixel_bicubic_wrapper (x, y, pixel, wrap, *this);
+}
+
+
+
+void
+ImageBuf::interppixel_bicubic_NDC (float x, float y, float *pixel, WrapMode wrap) const
+{
+    const ImageSpec &spec (impl()->spec());
+    interppixel_bicubic (static_cast<float>(spec.full_x) + x * static_cast<float>(spec.full_width),
+                         static_cast<float>(spec.full_y) + y * static_cast<float>(spec.full_height),
+                         pixel, wrap);
+}
+
+
+
 template<typename T>
 inline void
 setpixel_ (ImageBuf &buf, int x, int y, int z, const float *data, int chans)
@@ -1417,7 +1537,7 @@ ImageBuf::setpixel (int i, const float *pixel, int maxchannels)
 
 
 template<typename D, typename S>
-static inline bool 
+static bool
 get_pixel_channels_ (const ImageBuf &buf, int xbegin, int xend,
                      int ybegin, int yend, int zbegin, int zend, 
                      int chbegin, int chend, void *r_,
@@ -1448,10 +1568,12 @@ ImageBuf::get_pixel_channels (int xbegin, int xend, int ybegin, int yend,
                               stride_t xstride, stride_t ystride,
                               stride_t zstride) const
 {
-    OIIO_DISPATCH_TYPES2_HELP ("get_pixel_channels", get_pixel_channels_,
+    bool ok;
+    OIIO_DISPATCH_TYPES2_HELP (ok, "get_pixel_channels", get_pixel_channels_,
                                D, spec().format, *this,
                                xbegin, xend, ybegin, yend, zbegin, zend,
                                chbegin, chend, r, xstride, ystride, zstride);
+    return ok;
 }
 
 
@@ -1463,10 +1585,12 @@ ImageBuf::get_pixel_channels (int xbegin, int xend, int ybegin, int yend,
                               stride_t xstride, stride_t ystride,
                               stride_t zstride) const
 {
-    OIIO_DISPATCH_TYPES2 ("get_pixel_channels", get_pixel_channels_,
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "get_pixel_channels", get_pixel_channels_,
                           format, spec().format, *this,
                           xbegin, xend, ybegin, yend, zbegin, zend,
                           chbegin, chend, result, xstride, ystride, zstride);
+    return ok;
 }
 
 
@@ -1535,6 +1659,57 @@ ImageBuf::deep_value (int x, int y, int z, int c, int s) const
 
 
 
+uint32_t
+ImageBuf::deep_value_uint (int x, int y, int z, int c, int s) const
+{
+    impl()->validate_pixels();
+    if (! deep())
+        return 0;
+    const ImageSpec &m_spec (spec());
+    x -= m_spec.x;  y -= m_spec.y;  z -= m_spec.z;
+    int p = (z * m_spec.height + y) * m_spec.width + x;
+    return impl()->m_deepdata.deep_value_uint (p, c, s);
+}
+
+
+
+void
+ImageBuf::set_deep_value (int x, int y, int z, int c, int s, float value)
+{
+    impl()->validate_pixels();
+    if (! deep())
+        return ;
+    const ImageSpec &m_spec (spec());
+    x -= m_spec.x;  y -= m_spec.y;  z -= m_spec.z;
+    int p = (z * m_spec.height + y) * m_spec.width + x;
+    return impl()->m_deepdata.set_deep_value (p, c, s, value);
+}
+
+
+
+void
+ImageBuf::set_deep_value_uint (int x, int y, int z, int c, int s, uint32_t value)
+{
+    impl()->validate_pixels();
+    if (! deep())
+        return;
+    const ImageSpec &m_spec (spec());
+    x -= m_spec.x;  y -= m_spec.y;  z -= m_spec.z;
+    int p = (z * m_spec.height + y) * m_spec.width + x;
+    return impl()->m_deepdata.set_deep_value_uint (p, c, s, value);
+}
+
+
+
+void
+ImageBuf::deep_alloc ()
+{
+    deepdata()->alloc ();
+    m_impl->m_storage = ImageBuf::LOCALBUFFER;
+}
+
+
+
 int
 ImageBuf::xbegin () const
 {
@@ -1635,7 +1810,7 @@ ImageBuf::oriented_width () const
 {
     const ImageBufImpl *impl (this->impl());
     const ImageSpec &spec (impl->spec());
-    return impl->m_orientation <= 4 ? spec.width : spec.height;
+    return orientation() <= 4 ? spec.width : spec.height;
 }
 
 
@@ -1645,7 +1820,7 @@ ImageBuf::oriented_height () const
 {
     const ImageBufImpl *impl (this->impl());
     const ImageSpec &spec (impl->spec());
-    return impl->m_orientation <= 4 ? spec.height : spec.width;
+    return orientation() <= 4 ? spec.height : spec.width;
 }
 
 
@@ -1655,7 +1830,7 @@ ImageBuf::oriented_x () const
 {
     const ImageBufImpl *impl (this->impl());
     const ImageSpec &spec (impl->spec());
-    return impl->m_orientation <= 4 ? spec.x : spec.y;
+    return orientation() <= 4 ? spec.x : spec.y;
 }
 
 
@@ -1665,7 +1840,7 @@ ImageBuf::oriented_y () const
 {
     const ImageBufImpl *impl (this->impl());
     const ImageSpec &spec (impl->spec());
-    return impl->m_orientation <= 4 ? spec.y : spec.x;
+    return orientation() <= 4 ? spec.y : spec.x;
 }
 
 
@@ -1675,7 +1850,7 @@ ImageBuf::oriented_full_width () const
 {
     const ImageBufImpl *impl (this->impl());
     const ImageSpec &spec (impl->spec());
-    return impl->m_orientation <= 4 ? spec.full_width : spec.full_height;
+    return orientation() <= 4 ? spec.full_width : spec.full_height;
 }
 
 
@@ -1685,7 +1860,7 @@ ImageBuf::oriented_full_height () const
 {
     const ImageBufImpl *impl (this->impl());
     const ImageSpec &spec (impl->spec());
-    return impl->m_orientation <= 4 ? spec.full_height : spec.full_width;
+    return orientation() <= 4 ? spec.full_height : spec.full_width;
 }
 
 
@@ -1695,7 +1870,7 @@ ImageBuf::oriented_full_x () const
 {
     const ImageBufImpl *impl (this->impl());
     const ImageSpec &spec (impl->spec());
-    return impl->m_orientation <= 4 ? spec.full_x : spec.full_y;
+    return orientation() <= 4 ? spec.full_x : spec.full_y;
 }
 
 
@@ -1705,7 +1880,7 @@ ImageBuf::oriented_full_y () const
 {
     const ImageBufImpl *impl (this->impl());
     const ImageSpec &spec (impl->spec());
-    return impl->m_orientation <= 4 ? spec.full_y : spec.full_x;
+    return orientation() <= 4 ? spec.full_y : spec.full_x;
 }
 
 
@@ -1725,25 +1900,6 @@ ImageBuf::set_full (int xbegin, int xend, int ybegin, int yend,
 
 
 
-void
-ImageBuf::set_full (int xbegin, int xend, int ybegin, int yend,
-                    int zbegin, int zend, const float *bordercolor)
-{
-    ImageSpec &m_spec (impl()->specmod());
-    m_spec.full_x = xbegin;
-    m_spec.full_y = ybegin;
-    m_spec.full_z = zbegin;
-    m_spec.full_width  = xend - xbegin;
-    m_spec.full_height = yend - ybegin;
-    m_spec.full_depth  = zend - zbegin;
-    if (bordercolor)
-        m_spec.attribute ("oiio:bordercolor",
-                          TypeDesc(TypeDesc::FLOAT,m_spec.nchannels),
-                          bordercolor);
-}
-
-
-
 ROI
 ImageBuf::roi () const
 {
@@ -1917,6 +2073,8 @@ ImageBufImpl::retile (int x, int y, int z, ImageCache::Tile* &tile,
                                        m_current_miplevel, x, y, z);
         if (! tile) {
             // Even though tile is NULL, ensure valid black pixel data
+            std::string e = m_imagecache->geterror();
+            error ("%s", e.size() ? e : "unspecified ImageCache error");
             return &m_blackpixel[0];
         }
     }
