@@ -97,9 +97,10 @@ ImageBufAlgo::paste (ImageBuf &dst, int xbegin, int ybegin,
     IBAprep (dstroi, &dst);
 
     // do the actual copying
-    OIIO_DISPATCH_TYPES2 ("paste", paste_, dst.spec().format, src.spec().format,
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "paste", paste_, dst.spec().format, src.spec().format,
                           dst, dstroi_save, src, srcroi, nthreads);
-    return false;
+    return ok;
 }
 
 
@@ -138,9 +139,10 @@ ImageBufAlgo::crop (ImageBuf &dst, const ImageBuf &src,
     dst.clear ();
     roi.chend = std::min (roi.chend, src.nchannels());
     IBAprep (roi, &dst, &src);
-    OIIO_DISPATCH_TYPES2 ("crop", crop_, dst.spec().format, src.spec().format,
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "crop", crop_, dst.spec().format, src.spec().format,
                           dst, src, roi, nthreads);
-    return false;
+    return ok;
 }
 
 
@@ -167,13 +169,16 @@ ImageBufAlgo::cut (ImageBuf &dst, const ImageBuf &src,
 
 template<class D, class S>
 static bool
-flip_ (ImageBuf &dst, const ImageBuf &src, ROI roi, int nthreads)
+flip_ (ImageBuf &dst, const ImageBuf &src, ROI dst_roi, int nthreads)
 {
-    ImageBuf::ConstIterator<S, D> s (src, roi);
-    ImageBuf::Iterator<D, D> d (dst, roi);
+    ROI src_roi_full = src.roi_full();
+    ROI dst_roi_full = dst.roi_full();
+    ImageBuf::ConstIterator<S, D> s (src);
+    ImageBuf::Iterator<D, D> d (dst, dst_roi);
     for ( ; ! d.done(); ++d) {
-        s.pos (d.x(), roi.yend-1 - (d.y() - roi.ybegin), d.z());
-        for (int c = roi.chbegin; c < roi.chend; ++c)
+        int yy = d.y() - dst_roi_full.ybegin;
+        s.pos (d.x(), src_roi_full.yend-1 - yy, d.z());
+        for (int c = dst_roi.chbegin; c < dst_roi.chend; ++c)
             d[c] = s[c];
     }
     return true;
@@ -183,24 +188,47 @@ flip_ (ImageBuf &dst, const ImageBuf &src, ROI roi, int nthreads)
 bool
 ImageBufAlgo::flip(ImageBuf &dst, const ImageBuf &src, ROI roi, int nthreads)
 {
-    IBAprep (roi, &dst, &src);
-    OIIO_DISPATCH_TYPES2 ("flip", flip_,
-                          dst.spec().format, src.spec().format, dst, src,
-                          roi, nthreads);
-    return false;
+    if (&dst == &src) {    // Handle in-place operation
+        ImageBuf tmp;
+        tmp.swap (const_cast<ImageBuf&>(src));
+        return flip (dst, tmp, roi, nthreads);
+    }
+
+    ROI src_roi = roi.defined() ? roi : src.roi();
+    ROI src_roi_full = src.roi_full();
+    int offset = src_roi.ybegin - src_roi_full.ybegin;
+    int start = src_roi_full.yend - offset - src_roi.height();
+    ROI dst_roi (src_roi.xbegin, src_roi.xend,
+                 start, start+src_roi.height(),
+                 src_roi.zbegin, src_roi.zend,
+                 src_roi.chbegin, src_roi.chend);
+    ASSERT (dst_roi.width() == src_roi.width() &&
+            dst_roi.height() == src_roi.height());
+
+    // Compute the destination ROI, it's the source ROI reflected across
+    // the midline of the display window.
+    IBAprep (dst_roi, &dst, &src);
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "flip", flip_,
+                          dst.spec().format, src.spec().format,
+                          dst, src, dst_roi, nthreads);
+    return ok;
 }
 
 
 
 template<class D, class S>
 static bool
-flop_ (ImageBuf &dst, const ImageBuf &src, ROI roi, int nthreads)
+flop_ (ImageBuf &dst, const ImageBuf &src, ROI dst_roi, int nthreads)
 {
-    ImageBuf::ConstIterator<S, D> s (src, roi);
-    ImageBuf::Iterator<D, D> d (dst, roi);
+    ROI src_roi_full = src.roi_full();
+    ROI dst_roi_full = dst.roi_full();
+    ImageBuf::ConstIterator<S, D> s (src);
+    ImageBuf::Iterator<D, D> d (dst, dst_roi);
     for ( ; ! d.done(); ++d) {
-        s.pos (roi.xend-1 - (d.x() - roi.xbegin), d.y(), d.z());
-        for (int c = roi.chbegin; c < roi.chend; ++c)
+        int xx = d.x() - dst_roi_full.xbegin;
+        s.pos (src_roi_full.xend-1 - xx, d.y(), d.z());
+        for (int c = dst_roi.chbegin; c < dst_roi.chend; ++c)
             d[c] = s[c];
     }
     return true;
@@ -208,43 +236,263 @@ flop_ (ImageBuf &dst, const ImageBuf &src, ROI roi, int nthreads)
 
 
 bool
-ImageBufAlgo::flop (ImageBuf &dst, const ImageBuf &src, ROI roi, int nthreads)
+ImageBufAlgo::flop(ImageBuf &dst, const ImageBuf &src, ROI roi, int nthreads)
 {
-    IBAprep (roi, &dst, &src);
-    OIIO_DISPATCH_TYPES2 ("flop", flop_,
-                          dst.spec().format, src.spec().format, dst, src,
-                          roi, nthreads);
-    return false;
+    if (&dst == &src) {    // Handle in-place operation
+        ImageBuf tmp;
+        tmp.swap (const_cast<ImageBuf&>(src));
+        return flop (dst, tmp, roi, nthreads);
+    }
+
+    ROI src_roi = roi.defined() ? roi : src.roi();
+    ROI src_roi_full = src.roi_full();
+    int offset = src_roi.xbegin - src_roi_full.xbegin;
+    int start = src_roi_full.xend - offset - src_roi.width();
+    ROI dst_roi (start, start+src_roi.width(),
+                 src_roi.ybegin, src_roi.yend,
+                 src_roi.zbegin, src_roi.zend,
+                 src_roi.chbegin, src_roi.chend);
+    ASSERT (dst_roi.width() == src_roi.width() &&
+            dst_roi.height() == src_roi.height());
+
+    // Compute the destination ROI, it's the source ROI reflected across
+    // the midline of the display window.
+    IBAprep (dst_roi, &dst, &src);
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "flop", flop_,
+                          dst.spec().format, src.spec().format,
+                          dst, src, dst_roi, nthreads);
+    return ok;
 }
 
 
 
 template<class D, class S>
 static bool
-flipflop_ (ImageBuf &dst, const ImageBuf &src, ROI roi, int nthreads)
+rotate90_ (ImageBuf &dst, const ImageBuf &src, ROI dst_roi, int nthreads)
 {
-    // Serial case
-    ImageBuf::ConstIterator<S, D> s (src, roi);
-    ImageBuf::Iterator<D, D> d (dst, roi);
-    for ( ; !d.done(); ++d) {
-        s.pos (roi.xend-1 - (d.x() - roi.xbegin),
-               roi.yend-1 - (d.y() - roi.ybegin), d.z());
-        for (int c = roi.chbegin; c < roi.chend; ++c)
+    ROI dst_roi_full = dst.roi_full();
+    ImageBuf::ConstIterator<S, D> s (src);
+    ImageBuf::Iterator<D, D> d (dst, dst_roi);
+    for ( ; ! d.done(); ++d) {
+        s.pos (d.y(),
+               dst_roi_full.xend - d.x() - 1,
+               d.z());
+        for (int c = dst_roi.chbegin; c < dst_roi.chend; ++c)
             d[c] = s[c];
     }
     return true;
 }
 
 
+bool
+ImageBufAlgo::rotate90 (ImageBuf &dst, const ImageBuf &src,
+                        ROI roi, int nthreads)
+{
+    if (&dst == &src) {    // Handle in-place operation
+        ImageBuf tmp;
+        tmp.swap (const_cast<ImageBuf&>(src));
+        return rotate90 (dst, tmp, roi, nthreads);
+    }
+
+    ROI src_roi = roi.defined() ? roi : src.roi();
+    ROI src_roi_full = src.roi_full();
+
+    // Rotated full ROI swaps width and height, and keeps its origin
+    // where the original origin was.
+    ROI dst_roi_full (src_roi_full.xbegin, src_roi_full.xbegin+src_roi_full.height(),
+                      src_roi_full.ybegin, src_roi_full.ybegin+src_roi_full.width(),
+                      src_roi_full.zbegin, src_roi_full.zend,
+                      src_roi_full.chbegin, src_roi_full.chend);
+
+    ROI dst_roi (src_roi_full.yend-src_roi.yend,
+                 src_roi_full.yend-src_roi.ybegin,
+                 src_roi.xbegin, src_roi.xend,
+                 src_roi.zbegin, src_roi.zend,
+                 src_roi.chbegin, src_roi.chend);
+    ASSERT (dst_roi.width() == src_roi.height() &&
+            dst_roi.height() == src_roi.width());
+
+    bool dst_initialized = dst.initialized();
+    IBAprep (dst_roi, &dst, &src);
+    if (! dst_initialized)
+        dst.set_roi_full (dst_roi_full);
+
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "rotate90", rotate90_,
+                          dst.spec().format, src.spec().format,
+                          dst, src, dst_roi, nthreads);
+    return ok;
+}
+
+
+
+template<class D, class S>
+static bool
+rotate180_ (ImageBuf &dst, const ImageBuf &src, ROI dst_roi, int nthreads)
+{
+    ROI src_roi_full = src.roi_full();
+    ROI dst_roi_full = dst.roi_full();
+    ImageBuf::ConstIterator<S, D> s (src);
+    ImageBuf::Iterator<D, D> d (dst, dst_roi);
+    for ( ; ! d.done(); ++d) {
+        int xx = d.x() - dst_roi_full.xbegin;
+        int yy = d.y() - dst_roi_full.ybegin;
+        s.pos (src_roi_full.xend-1 - xx,
+               src_roi_full.yend-1 - yy,
+               d.z());
+        for (int c = dst_roi.chbegin; c < dst_roi.chend; ++c)
+            d[c] = s[c];
+    }
+    return true;
+}
+
+
+bool
+ImageBufAlgo::rotate180 (ImageBuf &dst, const ImageBuf &src,
+                         ROI roi, int nthreads)
+{
+    if (&dst == &src) {    // Handle in-place operation
+        ImageBuf tmp;
+        tmp.swap (const_cast<ImageBuf&>(src));
+        return rotate180 (dst, tmp, roi, nthreads);
+    }
+
+    ROI src_roi = roi.defined() ? roi : src.roi();
+    ROI src_roi_full = src.roi_full();
+    int xoffset = src_roi.xbegin - src_roi_full.xbegin;
+    int xstart = src_roi_full.xend - xoffset - src_roi.width();
+    int yoffset = src_roi.ybegin - src_roi_full.ybegin;
+    int ystart = src_roi_full.yend - yoffset - src_roi.height();
+    ROI dst_roi (xstart, xstart+src_roi.width(),
+                 ystart, ystart+src_roi.height(),
+                 src_roi.zbegin, src_roi.zend,
+                 src_roi.chbegin, src_roi.chend);
+    ASSERT (dst_roi.width() == src_roi.width() &&
+            dst_roi.height() == src_roi.height());
+
+    // Compute the destination ROI, it's the source ROI reflected across
+    // the midline of the display window.
+    IBAprep (dst_roi, &dst, &src);
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "rotate180", rotate180_,
+                          dst.spec().format, src.spec().format,
+                          dst, src, dst_roi, nthreads);
+    return ok;
+}
+
+
 bool
 ImageBufAlgo::flipflop (ImageBuf &dst, const ImageBuf &src,
                         ROI roi, int nthreads)
 {
-    IBAprep (roi, &dst, &src);
-    OIIO_DISPATCH_TYPES2 ("flipflop", flipflop_,
-                          dst.spec().format, src.spec().format, dst, src,
-                          roi, nthreads);
-    return false;
+    return rotate180 (dst, src, roi, nthreads);
+}
+
+
+
+template<class D, class S>
+static bool
+rotate270_ (ImageBuf &dst, const ImageBuf &src, ROI dst_roi, int nthreads)
+{
+    ROI dst_roi_full = dst.roi_full();
+    ImageBuf::ConstIterator<S, D> s (src);
+    ImageBuf::Iterator<D, D> d (dst, dst_roi);
+    for ( ; ! d.done(); ++d) {
+        s.pos (dst_roi_full.yend - d.y() - 1,
+               d.x(),
+               d.z());
+        for (int c = dst_roi.chbegin; c < dst_roi.chend; ++c)
+            d[c] = s[c];
+    }
+    return true;
+}
+
+
+bool
+ImageBufAlgo::rotate270 (ImageBuf &dst, const ImageBuf &src,
+                         ROI roi, int nthreads)
+{
+    if (&dst == &src) {    // Handle in-place operation
+        ImageBuf tmp;
+        tmp.swap (const_cast<ImageBuf&>(src));
+        return rotate270 (dst, tmp, roi, nthreads);
+    }
+
+    ROI src_roi = roi.defined() ? roi : src.roi();
+    ROI src_roi_full = src.roi_full();
+
+    // Rotated full ROI swaps width and height, and keeps its origin
+    // where the original origin was.
+    ROI dst_roi_full (src_roi_full.xbegin, src_roi_full.xbegin+src_roi_full.height(),
+                      src_roi_full.ybegin, src_roi_full.ybegin+src_roi_full.width(),
+                      src_roi_full.zbegin, src_roi_full.zend,
+                      src_roi_full.chbegin, src_roi_full.chend);
+
+    ROI dst_roi (src_roi.ybegin, src_roi.yend,
+                 src_roi_full.xend-src_roi.xend,
+                 src_roi_full.xend-src_roi.xbegin,
+                 src_roi.zbegin, src_roi.zend,
+                 src_roi.chbegin, src_roi.chend);
+
+    ASSERT (dst_roi.width() == src_roi.height() &&
+            dst_roi.height() == src_roi.width());
+
+    bool dst_initialized = dst.initialized();
+    IBAprep (dst_roi, &dst, &src);
+    if (! dst_initialized)
+        dst.set_roi_full (dst_roi_full);
+
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "rotate270", rotate270_,
+                          dst.spec().format, src.spec().format,
+                          dst, src, dst_roi, nthreads);
+    return ok;
+}
+
+
+
+bool
+ImageBufAlgo::reorient (ImageBuf &dst, const ImageBuf &src, int nthreads)
+{
+    ImageBuf tmp;
+    bool ok = false;
+    switch (src.orientation()) {
+    case 1:
+        ok = dst.copy (src);
+        break;
+    case 2:
+        ok = ImageBufAlgo::flop (dst, src);
+        break;
+    case 3:
+        ok = ImageBufAlgo::rotate180 (dst, src);
+        break;
+    case 4:
+        ok = ImageBufAlgo::flip (dst, src);
+        break;
+    case 5:
+        ok = ImageBufAlgo::rotate270 (tmp, src);
+        if (ok)
+            ok = ImageBufAlgo::flop (dst, tmp);
+        else
+            dst.error ("%s", tmp.geterror());
+        break;
+    case 6:
+        ok = ImageBufAlgo::rotate90 (dst, src);
+        break;
+    case 7:
+        ok = ImageBufAlgo::flip (tmp, src);
+        if (ok)
+            ok = ImageBufAlgo::rotate90 (dst, tmp);
+        else
+            dst.error ("%s", tmp.geterror());
+        break;
+    case 8:
+        ok = ImageBufAlgo::rotate270 (dst, src);
+        break;
+    }
+    dst.set_orientation (1);
+    return ok;
 }
 
 
@@ -288,10 +536,18 @@ ImageBufAlgo::transpose (ImageBuf &dst, const ImageBuf &src,
     roi.chend = std::min (roi.chend, src.nchannels());
     ROI dst_roi (roi.ybegin, roi.yend, roi.xbegin, roi.xend,
                  roi.zbegin, roi.zend, roi.chbegin, roi.chend);
+    bool dst_initialized = dst.initialized();
     IBAprep (dst_roi, &dst);
-    OIIO_DISPATCH_TYPES2 ("transpose", transpose_, dst.spec().format,
+    if (! dst_initialized) {
+        ROI r = src.roi_full();
+        ROI dst_roi_full (r.ybegin, r.yend, r.xbegin, r.xend,
+                          r.zbegin, r.zend, r.chbegin, r.chend);
+        dst.set_roi_full (dst_roi_full);
+    }
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "transpose", transpose_, dst.spec().format,
                           src.spec().format, dst, src, roi, nthreads);
-    return false;
+    return ok;
 }
 
 
@@ -338,10 +594,11 @@ ImageBufAlgo::circular_shift (ImageBuf &dst, const ImageBuf &src,
                               ROI roi, int nthreads)
 {
     IBAprep (roi, &dst, &src);
-    OIIO_DISPATCH_TYPES2 ("circular_shift", circular_shift_,
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "circular_shift", circular_shift_,
                           dst.spec().format, src.spec().format, dst, src,
                           xshift, yshift, zshift, roi, roi, nthreads);
-    return false;
+    return ok;
 }
 
 
@@ -387,8 +644,10 @@ ImageBufAlgo::channels (ImageBuf &dst, const ImageBuf &src,
     ImageSpec newspec = src.spec();
     newspec.nchannels = nchannels;
     newspec.default_channel_names ();
+    newspec.channelformats.clear();
     newspec.alpha_channel = -1;
     newspec.z_channel = -1;
+    bool all_same_type = true;
     for (int c = 0; c < nchannels;  ++c) {
         int csrc = channelorder[c];
         // If the user gave an explicit name for this channel, use it...
@@ -403,6 +662,10 @@ ImageBufAlgo::channels (ImageBuf &dst, const ImageBuf &src,
         else if (csrc >= 0 && csrc < src.spec().nchannels) {
             newspec.channelnames[c] = src.spec().channelnames[csrc];
         }
+        TypeDesc type = src.spec().channelformat(csrc);
+        newspec.channelformats.push_back (type);
+        if (type != newspec.channelformats.front())
+            all_same_type = false;
         // Use the names (or designation of the src image, if
         // shuffle_channel_names is true) to deduce the alpha and z channels.
         if ((shuffle_channel_names && csrc == src.spec().alpha_channel) ||
@@ -413,9 +676,49 @@ ImageBufAlgo::channels (ImageBuf &dst, const ImageBuf &src,
               Strutil::iequals (newspec.channelnames[c], "Z"))
             newspec.z_channel = c;
     }
+    if (all_same_type)                      // clear per-chan formats if
+        newspec.channelformats.clear();     // they're all the same
 
     // Update the image (realloc with the new spec)
-    dst.alloc (newspec);
+    dst.reset (newspec);
+
+    if (dst.deep()) {
+        // Deep case:
+        ASSERT (src.deep() && src.deepdata() && dst.deepdata());
+        const DeepData &srcdata (*src.deepdata());
+        DeepData &dstdata (*dst.deepdata());
+        // The earlier dst.alloc() already called dstdata.init()
+        dstdata.nsamples = srcdata.nsamples;
+        dst.deep_alloc ();
+        for (int p = 0, npels = (int)newspec.image_pixels(); p < npels; ++p) {
+            if (! dstdata.nsamples[p])
+                continue;   // no samples for this pixel
+            for (int c = 0;  c < newspec.nchannels;  ++c) {
+                int csrc = channelorder[c];
+                if (csrc < 0) {
+                    // Replacing the channel with a new value
+                    float val = channelvalues ? channelvalues[c] : 0.0f;
+                    for (int s = 0, ns = dstdata.nsamples[p]; s < ns; ++s)
+                        dstdata.set_deep_value (p, c, s, val);
+                } else if (dstdata.channeltypes[c] == srcdata.channeltypes[csrc]) {
+                    // Same channel types -- copy all samples at once
+                    memcpy (dstdata.channel_ptr(p,c), srcdata.channel_ptr(p,csrc),
+                            dstdata.nsamples[p] * srcdata.channeltypes[csrc].size());
+                } else {
+                    if (dstdata.channeltypes[c] == TypeDesc::UINT)
+                        for (int s = 0, ns = dstdata.nsamples[p]; s < ns; ++s)
+                            dstdata.set_deep_value_uint (p, c, s,
+                                          srcdata.deep_value_uint(p,csrc,s));
+                    else
+                        for (int s = 0, ns = dstdata.nsamples[p]; s < ns; ++s)
+                            dstdata.set_deep_value (p, c, s,
+                                          srcdata.deep_value(p,csrc,s));
+                }
+            }
+        }
+        return true;
+    }
+    // Below is the non-deep case
 
     // Copy the channels individually
     stride_t dstxstride = AutoStride, dstystride = AutoStride, dstzstride = AutoStride;
@@ -522,9 +825,10 @@ ImageBufAlgo::channel_append (ImageBuf &dst, const ImageBuf &A,
         return false;
     }
 
-    OIIO_DISPATCH_TYPES ("channel_append", channel_append_impl,
+    bool ok;
+    OIIO_DISPATCH_TYPES (ok, "channel_append", channel_append_impl,
                          A.spec().format, dst, A, B, roi, nthreads);
-    return true;
+    return ok;
 }
 
 
