@@ -44,8 +44,6 @@
 #include "OpenImageIO/imagebufalgo.h"
 #include "OpenImageIO/imagebufalgo_util.h"
 #include "OpenImageIO/dassert.h"
-#include "OpenImageIO/sysutil.h"
-#include "OpenImageIO/thread.h"
 
 
 
@@ -101,19 +99,11 @@ ImageBufAlgo::clamp (ImageBuf &dst, const ImageBuf &src,
         maxvec.resize (dst.nchannels(), std::numeric_limits<float>::max());
         max = &maxvec[0];
     }
-    OIIO_DISPATCH_TYPES2 ("clamp", clamp_, dst.spec().format,
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "clamp", clamp_, dst.spec().format,
                           src.spec().format, dst, src,
                           min, max, clampalpha01, roi, nthreads);
-    return false;
-}
-
-
-
-bool
-ImageBufAlgo::clamp (ImageBuf &dst, const float *min, const float *max,
-                     bool clampalpha01, ROI roi, int nthreads)
-{
-    return clamp (dst, dst, min, max, clampalpha01, roi, nthreads);
+    return ok;
 }
 
 
@@ -130,15 +120,6 @@ ImageBufAlgo::clamp (ImageBuf &dst, const ImageBuf &src,
 
 
 
-bool
-ImageBufAlgo::clamp (ImageBuf &dst, float min, float max,
-                     bool clampalpha01, ROI roi, int nthreads)
-{
-    return clamp (dst, dst, min, max, clampalpha01, roi, nthreads);
-}
-
-
-
 template<class Rtype, class Atype, class Btype>
 static bool
 add_impl (ImageBuf &R, const ImageBuf &A, const ImageBuf &B,
@@ -198,10 +179,11 @@ ImageBufAlgo::add (ImageBuf &dst, const ImageBuf &A, const ImageBuf &B,
 {
     if (! IBAprep (roi, &dst, &A, &B))
         return false;
-    OIIO_DISPATCH_COMMON_TYPES3 ("add", add_impl, dst.spec().format,
+    bool ok;
+    OIIO_DISPATCH_COMMON_TYPES3 (ok, "add", add_impl, dst.spec().format,
                                  A.spec().format, B.spec().format,
                                  dst, A, B, roi, nthreads);
-    return true;
+    return ok;
 }
 
 
@@ -212,9 +194,10 @@ ImageBufAlgo::add (ImageBuf &dst, const ImageBuf &A, const float *b,
 {
     if (! IBAprep (roi, &dst, &A))
         return false;
-    OIIO_DISPATCH_TYPES2 ("add", add_impl, dst.spec().format,
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "add", add_impl, dst.spec().format,
                           A.spec().format, dst, A, b, roi, nthreads);
-    return true;
+    return ok;
 }
 
 
@@ -229,24 +212,10 @@ ImageBufAlgo::add (ImageBuf &dst, const ImageBuf &A, float b,
     float *vals = ALLOCA (float, nc);
     for (int c = 0;  c < nc;  ++c)
         vals[c] = b;
-    OIIO_DISPATCH_TYPES2 ("add", add_impl, dst.spec().format,
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "add", add_impl, dst.spec().format,
                           A.spec().format, dst, A, vals, roi, nthreads);
-}
-
-
-
-bool
-ImageBufAlgo::add (ImageBuf &dst, const float *val, ROI roi, int nthreads)
-{
-    return add (dst, dst, val, roi, nthreads);
-}
-
-
-
-bool
-ImageBufAlgo::add (ImageBuf &R, float val, ROI roi, int nthreads)
-{
-    return add (R, R, val, roi, nthreads);
+    return ok;
 }
 
 
@@ -285,10 +254,11 @@ ImageBufAlgo::sub (ImageBuf &dst, const ImageBuf &A, const ImageBuf &B,
 {
     if (! IBAprep (roi, &dst, &A, &B))
         return false;
-    OIIO_DISPATCH_COMMON_TYPES3 ("sub", sub_impl, dst.spec().format,
+    bool ok;
+    OIIO_DISPATCH_COMMON_TYPES3 (ok, "sub", sub_impl, dst.spec().format,
                                  A.spec().format, B.spec().format,
                                  dst, A, B, roi, nthreads);
-    return true;
+    return ok;
 }
 
 
@@ -303,9 +273,10 @@ ImageBufAlgo::sub (ImageBuf &dst, const ImageBuf &A, const float *b,
     float *vals = ALLOCA (float, nc);
     for (int c = 0;  c < nc;  ++c)
         vals[c] = -b[c];
-    OIIO_DISPATCH_TYPES2 ("sub", add_impl, dst.spec().format,
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "sub", add_impl, dst.spec().format,
                           A.spec().format, dst, A, vals, roi, nthreads);
-    return true;
+    return ok;
 }
 
 
@@ -320,8 +291,10 @@ ImageBufAlgo::sub (ImageBuf &dst, const ImageBuf &A, float b,
     float *vals = ALLOCA (float, nc);
     for (int c = 0;  c < nc;  ++c)
         vals[c] = -b;
-    OIIO_DISPATCH_TYPES2 ("sub", add_impl, dst.spec().format,
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "sub", add_impl, dst.spec().format,
                           A.spec().format, dst, A, vals, roi, nthreads);
+    return ok;
 }
 
 
@@ -359,10 +332,11 @@ ImageBufAlgo::mul (ImageBuf &dst, const ImageBuf &A, const ImageBuf &B,
 {
     if (! IBAprep (roi, &dst, &A, &B))
         return false;
-    OIIO_DISPATCH_COMMON_TYPES3 ("mul", mul_impl, dst.spec().format,
+    bool ok;
+    OIIO_DISPATCH_COMMON_TYPES3 (ok, "mul", mul_impl, dst.spec().format,
                                  A.spec().format, B.spec().format,
                                  dst, A, B, roi, nthreads);
-    return true;
+    return ok;
 }
 
 
@@ -395,9 +369,10 @@ ImageBufAlgo::mul (ImageBuf &dst, const ImageBuf &A, const float *b,
 {
     if (! IBAprep (roi, &dst, &A))
         return false;
-    OIIO_DISPATCH_TYPES2 ("mul", mul_impl, dst.spec().format,
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "mul", mul_impl, dst.spec().format,
                           A.spec().format, dst, A, b, roi, nthreads);
-    return true;
+    return ok;
 }
 
 
@@ -412,29 +387,15 @@ ImageBufAlgo::mul (ImageBuf &dst, const ImageBuf &A, float b,
     float *vals = ALLOCA (float, nc);
     for (int c = 0;  c < nc;  ++c)
         vals[c] = b;
-    OIIO_DISPATCH_TYPES2 ("mul", mul_impl, dst.spec().format,
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "mul", mul_impl, dst.spec().format,
                           A.spec().format, dst, A, vals, roi, nthreads);
+    return ok;
 }
 
 
 
 
-bool
-ImageBufAlgo::mul (ImageBuf &dst, const float *val, ROI roi, int nthreads)
-{
-    return mul (dst, dst, val, roi, nthreads);
-}
-
-
-
-bool
-ImageBufAlgo::mul (ImageBuf &dst, float val, ROI roi, int nthreads)
-{
-    return mul (dst, dst, val, roi, nthreads);
-}
-
-
-
 template<class Rtype, class Atype>
 static bool
 pow_impl (ImageBuf &R, const ImageBuf &A, const float *b,
@@ -463,9 +424,10 @@ ImageBufAlgo::pow (ImageBuf &dst, const ImageBuf &A, const float *b,
 {
     if (! IBAprep (roi, &dst, &A))
         return false;
-    OIIO_DISPATCH_TYPES2 ("pow", pow_impl, dst.spec().format,
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "pow", pow_impl, dst.spec().format,
                           A.spec().format, dst, A, b, roi, nthreads);
-    return true;
+    return ok;
 }
 
 
@@ -480,8 +442,10 @@ ImageBufAlgo::pow (ImageBuf &dst, const ImageBuf &A, float b,
     float *vals = ALLOCA (float, nc);
     for (int c = 0;  c < nc;  ++c)
         vals[c] = b;
-    OIIO_DISPATCH_TYPES2 ("pow", pow_impl, dst.spec().format,
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "pow", pow_impl, dst.spec().format,
                           A.spec().format, dst, A, vals, roi, nthreads);
+    return ok;
 }
 
 
@@ -535,10 +499,11 @@ ImageBufAlgo::channel_sum (ImageBuf &dst, const ImageBuf &src,
         weights = &local_weights[0];
     }
 
-    OIIO_DISPATCH_TYPES2 ("channel_sum", channel_sum_,
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "channel_sum", channel_sum_,
                           dst.spec().format, src.spec().format,
                           dst, src, weights, roi, nthreads);
-    return false;
+    return ok;
 }
 
 
@@ -618,7 +583,7 @@ rangecompress_ (ImageBuf &R, const ImageBuf &A,
         for (ImageBuf::Iterator<Rtype> r (R, roi);  !r.done();  ++r) {
             if (useluma) {
                 float luma = 0.21264f * r[roi.chbegin] + 0.71517f * r[roi.chbegin+1] + 0.07219f * r[roi.chbegin+2];
-                float scale = rangecompress (luma) / luma;
+                float scale = luma > 0.0f ? rangecompress (luma) / luma : 0.0f;
                 for (int c = roi.chbegin; c < roi.chend; ++c) {
                     if (c == alpha_channel || c == z_channel)
                         continue;
@@ -636,8 +601,8 @@ rangecompress_ (ImageBuf &R, const ImageBuf &A,
         ImageBuf::ConstIterator<Atype> a (A, roi);
         for (ImageBuf::Iterator<Rtype> r (R, roi);  !r.done();  ++r, ++a) {
             if (useluma) {
-                float luma = 0.21264f * r[roi.chbegin] + 0.71517f * r[roi.chbegin+1] + 0.07219f * r[roi.chbegin+2];
-                float scale = rangecompress (luma) / luma;
+                float luma = 0.21264f * a[roi.chbegin] + 0.71517f * a[roi.chbegin+1] + 0.07219f * a[roi.chbegin+2];
+                float scale = luma > 0.0f ? rangecompress (luma) / luma : 0.0f;
                 for (int c = roi.chbegin; c < roi.chend; ++c) {
                     if (c == alpha_channel || c == z_channel)
                         r[c] = a[c];
@@ -688,7 +653,7 @@ rangeexpand_ (ImageBuf &R, const ImageBuf &A,
         for (ImageBuf::Iterator<Rtype> r (R, roi);  !r.done();  ++r) {
             if (useluma) {
                 float luma = 0.21264f * r[roi.chbegin] + 0.71517f * r[roi.chbegin+1] + 0.07219f * r[roi.chbegin+2];
-                float scale = rangeexpand (luma) / luma;
+                float scale = luma > 0.0f ? rangeexpand (luma) / luma : 0.0f;
                 for (int c = roi.chbegin; c < roi.chend; ++c) {
                     if (c == alpha_channel || c == z_channel)
                         continue;
@@ -706,8 +671,8 @@ rangeexpand_ (ImageBuf &R, const ImageBuf &A,
         ImageBuf::ConstIterator<Atype> a (A, roi);
         for (ImageBuf::Iterator<Rtype> r (R, roi);  !r.done();  ++r, ++a) {
             if (useluma) {
-                float luma = 0.21264f * r[roi.chbegin] + 0.71517f * r[roi.chbegin+1] + 0.07219f * r[roi.chbegin+2];
-                float scale = rangeexpand (luma) / luma;
+                float luma = 0.21264f * a[roi.chbegin] + 0.71517f * a[roi.chbegin+1] + 0.07219f * a[roi.chbegin+2];
+                float scale = luma > 0.0f ? rangeexpand (luma) / luma : 0.0f;
                 for (int c = roi.chbegin; c < roi.chend; ++c) {
                     if (c == alpha_channel || c == z_channel)
                         r[c] = a[c];
@@ -739,10 +704,11 @@ ImageBufAlgo::rangecompress (ImageBuf &dst, const ImageBuf &src,
     }
     if (! IBAprep (roi, &dst, &src))
         return false;
-    OIIO_DISPATCH_TYPES2 ("rangecompress", rangecompress_,
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "rangecompress", rangecompress_,
                           dst.spec().format, src.spec().format,
                           dst, src, useluma, roi, nthreads);
-    return true;
+    return ok;
 }
 
 
@@ -757,28 +723,11 @@ ImageBufAlgo::rangeexpand (ImageBuf &dst, const ImageBuf &src,
     }
     if (! IBAprep (roi, &dst))
         return false;
-    OIIO_DISPATCH_TYPES2 ("rangeexpand", rangeexpand_,
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "rangeexpand", rangeexpand_,
                           dst.spec().format, src.spec().format,
                           dst, src, useluma, roi, nthreads);
-    return true;
-}
-
-
-
-bool
-ImageBufAlgo::rangecompress (ImageBuf &dst, bool useluma,
-                             ROI roi, int nthreads)
-{
-    return rangecompress (dst, dst, useluma, roi, nthreads);
-}
-
-
-
-bool
-ImageBufAlgo::rangeexpand (ImageBuf &dst, bool useluma,
-                           ROI roi, int nthreads)
-{
-    return rangeexpand (dst, dst, useluma, roi, nthreads);
+    return ok;
 }
 
 
@@ -841,17 +790,10 @@ ImageBufAlgo::unpremult (ImageBuf &dst, const ImageBuf &src,
                           roi.chbegin, src, roi, nthreads);
         return true;
     }
-    OIIO_DISPATCH_TYPES2 ("unpremult", unpremult_, dst.spec().format,
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "unpremult", unpremult_, dst.spec().format,
                           src.spec().format, dst, src, roi, nthreads);
-    return true;
-}
-
-
-
-bool
-ImageBufAlgo::unpremult (ImageBuf &dst, ROI roi, int nthreads)
-{
-    return unpremult (dst, dst, roi, nthreads);
+    return ok;
 }
 
 
@@ -909,16 +851,10 @@ ImageBufAlgo::premult (ImageBuf &dst, const ImageBuf &src,
                           roi.chbegin, src, roi, nthreads);
         return true;
     }
-    OIIO_DISPATCH_TYPES2 ("premult", premult_, dst.spec().format,
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "premult", premult_, dst.spec().format,
                           src.spec().format, dst, src, roi, nthreads);
-    return true;
-}
-
-
-bool
-ImageBufAlgo::premult (ImageBuf &dst, ROI roi, int nthreads)
-{
-    return premult (dst, dst, roi, nthreads);
+    return ok;
 }
 
 
@@ -1059,17 +995,6 @@ ImageBufAlgo::fixNonFinite (ImageBuf &dst, const ImageBuf &src,
 
 
 
-// DEPRECATED (1.3)
-bool
-ImageBufAlgo::fixNonFinite (ImageBuf &dst,
-                            NonFiniteFixMode mode, int *pixelsFixed,
-                            ROI roi, int nthreads)
-{
-    return fixNonFinite (dst, dst, mode, pixelsFixed, roi, nthreads);
-}
-
-
-
 
 static bool
 decode_over_channels (const ImageBuf &R, int &nchannels, 
@@ -1172,10 +1097,11 @@ ImageBufAlgo::over (ImageBuf &dst, const ImageBuf &A, const ImageBuf &B,
     if (! IBAprep (roi, &dst, &A, &B, NULL,
                    IBAprep_REQUIRE_ALPHA | IBAprep_REQUIRE_SAME_NCHANNELS))
         return false;
-    OIIO_DISPATCH_COMMON_TYPES3 ("over", over_impl, dst.spec().format,
+    bool ok;
+    OIIO_DISPATCH_COMMON_TYPES3 (ok, "over", over_impl, dst.spec().format,
                                  A.spec().format, B.spec().format,
                                  dst, A, B, false, false, roi, nthreads);
-    return ! dst.has_error();
+    return ok && ! dst.has_error();
 }
 
 
@@ -1188,10 +1114,11 @@ ImageBufAlgo::zover (ImageBuf &dst, const ImageBuf &A, const ImageBuf &B,
                    IBAprep_REQUIRE_ALPHA | IBAprep_REQUIRE_Z |
                    IBAprep_REQUIRE_SAME_NCHANNELS))
         return false;
-    OIIO_DISPATCH_COMMON_TYPES3 ("zover", over_impl, dst.spec().format,
+    bool ok;
+    OIIO_DISPATCH_COMMON_TYPES3 (ok, "zover", over_impl, dst.spec().format,
                                  A.spec().format, B.spec().format,
                                  dst, A, B, true, z_zeroisinf, roi, nthreads);
-    return ! dst.has_error();
+    return ok && ! dst.has_error();
 }
 
 
