@@ -59,6 +59,8 @@ public:
     bool dumpdata_showempty;
     bool hash;
     bool updatemode;
+    bool autoorient;
+    bool nativeread;                  // force native data type reads
     int threads;
     std::string full_command_line;
     std::string printinfo_metamatch;
@@ -78,6 +80,7 @@ public:
     bool output_autotrim;
     bool output_dither;
     bool output_force_tiles; // for debugging
+    bool metadata_nosoftwareattrib;
 
     // Options for --diff
     float diff_warnthresh;
@@ -106,9 +109,13 @@ public:
     void clear_options ();
 
     // Force img to be read at this point.
-    void read (ImageRecRef img);
+    bool read (ImageRecRef img);
     // Read the current image
-    void read () { if (curimg) read (curimg); }
+    bool read () {
+        if (curimg)
+            return read (curimg);
+        return true;
+    }
 
     // If required_images are not yet on the stack, then postpone this
     // call by putting it on the 'pending' list and return true.
@@ -147,7 +154,18 @@ public:
 
     ImageRecRef top () { return curimg; }
 
-    void error (const std::string &command, const std::string &explanation="");
+    // Modify the resolution and/or offset according to what's in geom.
+    // Valid geometries are WxH (resolution), +X+Y (offsets), WxH+X+Y
+    // (resolution and offset).  If 'allow_scaling' is true, geometries of
+    // S% (e.g. "50%") or just S (e.g., "1.2") will be accepted to scale the
+    // existing width and height (rounding to the nearest whole number of
+    // pixels.
+    bool adjust_geometry (string_view command,
+                          int &w, int &h, int &x, int &y, const char *geom,
+                          bool allow_scaling=false);
+
+    void error (string_view command, string_view explanation="");
+    void warning (string_view command, string_view explanation="");
 
 private:
     CallbackFunction m_pending_callback;
@@ -267,7 +285,7 @@ public:
     // it's lazily kept as name only, without reading the file.)
     bool elaborated () const { return m_elaborated; }
 
-    bool read ();
+    bool read (bool force_native_read=false);
 
     // ir(subimg,mip) references a specific MIP level of a subimage
     // ir(subimg) references the first MIP level of a subimage
@@ -301,6 +319,21 @@ public:
         metadata_modified();
     }
 
+    /// Error reporting for ImageRec: call this with printf-like arguments.
+    /// Note however that this is fully typesafe!
+    /// void error (const char *format, ...)
+    TINYFORMAT_WRAP_FORMAT (void, error, const,
+        std::ostringstream msg;, msg, append_error(msg.str());)
+
+    /// Return true if the IR has had an error and has an error message
+    /// to retrieve via geterror().
+    bool has_error (void) const;
+
+    /// Return the text of all error messages issued since geterror() was
+    /// called (or an empty string if no errors are pending).  This also
+    /// clears the error message for next time if clear_error is true.
+    std::string geterror (bool clear_error = true) const;
+
 private:
     std::string m_name;
     bool m_elaborated;
@@ -309,6 +342,11 @@ private:
     std::vector<SubimageRec> m_subimages;
     std::time_t m_time;  //< Modification time of the input file
     ImageCache *m_imagecache;
+    mutable std::string m_err;
+
+    // Add to the error message
+    void append_error (string_view message) const;
+
 };
 
 
@@ -340,20 +378,11 @@ struct print_info_options {
 // of the uncompressed pixels in the file is returned in totalsize.  The
 // return value will be true if everything is ok, or false if there is
 // an error (in which case the error message will be stored in 'error').
-bool print_info (const std::string &filename, 
+bool print_info (Oiiotool &ot, const std::string &filename, 
                  const print_info_options &opt,
                  long long &totalsize, std::string &error);
 
 
-// Modify the resolution and/or offset according to what's in geom.
-// Valid geometries are WxH (resolution), +X+Y (offsets), WxH+X+Y
-// (resolution and offset).  If 'allow_scaling' is true, geometries of
-// S% (e.g. "50%") or just S (e.g., "1.2") will be accepted to scale the
-// existing width and height (rounding to the nearest whole number of
-// pixels.
-bool adjust_geometry (int &w, int &h, int &x, int &y, const char *geom,
-                      bool allow_scaling=false);
-
 // Set an attribute of the given image.  The type should be one of
 // TypeDesc::INT (decode the value as an int), FLOAT, STRING, or UNKNOWN
 // (look at the string and try to discern whether it's an int, float, or
@@ -396,7 +425,9 @@ bool apply_spec_mod (ImageRec &img, Action act, const Type &t,
     img.metadata_modified (true);
     for (int s = 0, send = img.subimages();  s < send;  ++s) {
         for (int m = 0, mend = img.miplevels(s);  m < mend;  ++m) {
-            ok &= act (*img.spec(s,m), t);
+            ok &= act (img(s,m).specmod(), t);
+            if (ok)
+                img.update_spec_from_imagebuf (s, m);
             if (! allsubimages)
                 break;
         }
