@@ -284,6 +284,10 @@ OpenEXROutput::supports (const std::string &feature) const
         return true;
     if (feature == "mipmap")
         return true;
+    if (feature == "alpha")
+        return true;
+    if (feature == "nchannels")
+        return true;
     if (feature == "channelformats")
         return true;
     if (feature == "displaywindow")
@@ -292,6 +296,12 @@ OpenEXROutput::supports (const std::string &feature) const
         return true;
     if (feature == "negativeorigin")
         return true;
+    if (feature == "arbitrary_metadata")
+        return true;
+    if (feature == "exif")   // Because of arbitrary_metadata
+        return true;
+    if (feature == "iptc")   // Because of arbitrary_metadata
+        return true;
 #ifdef USE_OPENEXR_VERSION2
     if (feature == "multiimage")
         return true;  // N.B. But OpenEXR does not support "appendsubimage"
@@ -339,12 +349,16 @@ OpenEXROutput::open (const std::string &name, const ImageSpec &userspec,
                 m_output_scanline = new Imf::OutputFile (*m_output_stream,
                                                          m_headers[m_subimage]);
             }
-        }
-        catch (const std::exception &e) {
+        } catch (const std::exception &e) {
             error ("OpenEXR exception: %s", e.what());
             m_output_scanline = NULL;
             m_output_tiled = NULL;
             return false;
+        } catch (...) {   // catch-all for edge cases or compiler bugs
+            error ("OpenEXR exception: unknown");
+            m_output_scanline = NULL;
+            m_output_tiled = NULL;
+            return false;
         }
         if (! m_output_scanline && ! m_output_tiled) {
             error ("Unknown error opening EXR file");
@@ -385,14 +399,20 @@ OpenEXROutput::open (const std::string &name, const ImageSpec &userspec,
             } else {
                 ASSERT (0);
             }
-        }
-        catch (const std::exception &e) {
+        } catch (const std::exception &e) {
             error ("OpenEXR exception: %s", e.what());
             m_scanline_output_part = NULL;
             m_tiled_output_part = NULL;
             m_deep_scanline_output_part = NULL;
             m_deep_tiled_output_part = NULL;
             return false;
+        } catch (...) {   // catch-all for edge cases or compiler bugs
+            error ("OpenEXR exception: unknown exception");
+            m_scanline_output_part = NULL;
+            m_tiled_output_part = NULL;
+            m_deep_scanline_output_part = NULL;
+            m_deep_tiled_output_part = NULL;
+            return false;
         }
         m_spec = m_subimagespecs[m_subimage];
         return true;
@@ -490,12 +510,16 @@ OpenEXROutput::open (const std::string &name, int subimages,
         // do this quite yet.
         m_output_multipart = new Imf::MultiPartOutputFile (name.c_str(),
                                                  &m_headers[0], subimages);
-    }
-    catch (const std::exception &e) {
+    } catch (const std::exception &e) {
         delete m_output_stream;
         m_output_stream = NULL;
         error ("OpenEXR exception: %s", e.what());
         return false;
+    } catch (...) {   // catch-all for edge cases or compiler bugs
+        delete m_output_stream;
+        m_output_stream = NULL;
+        error ("OpenEXR exception: unknown exception");
+        return false;
     }
     try {
         if (deep) {
@@ -511,8 +535,7 @@ OpenEXROutput::open (const std::string &name, int subimages,
                 m_scanline_output_part = new Imf::OutputPart (*m_output_multipart, 0);
             }
         }
-    }
-    catch (const std::exception &e) {
+    } catch (const std::exception &e) {
         error ("OpenEXR exception: %s", e.what());
         delete m_output_stream;  m_output_stream = NULL;
         m_scanline_output_part = NULL;
@@ -520,6 +543,14 @@ OpenEXROutput::open (const std::string &name, int subimages,
         m_deep_scanline_output_part = NULL;
         m_deep_tiled_output_part = NULL;
         return false;
+    } catch (...) {  // catch-all for edge cases or compiler bugs
+        error ("OpenEXR exception: unknown exception");
+        delete m_output_stream;  m_output_stream = NULL;
+        m_scanline_output_part = NULL;
+        m_tiled_output_part = NULL;
+        m_deep_scanline_output_part = NULL;
+        m_deep_tiled_output_part = NULL;
+        return false;
     }
 
     return true;
@@ -668,6 +699,25 @@ OpenEXROutput::spec_to_header (ImageSpec &spec, int subimage, Imf::Header &heade
         header.insert ("envmap", Imf::EnvmapAttribute(Imf::ENVMAP_LATLONG));
     }
 
+    // Fix up density and aspect to be consistent
+    float aspect = spec.get_float_attribute ("PixelAspectRatio", 0.0f);
+    float xdensity = spec.get_float_attribute ("XResolution", 0.0f);
+    float ydensity = spec.get_float_attribute ("XResolution", 0.0f);
+    if (! aspect && xdensity && ydensity) {
+        // No aspect ratio. Compute it from density, if supplied.
+        spec.attribute ("PixelAspectRatio", xdensity / ydensity);
+    }
+    if (xdensity && ydensity &&
+            spec.get_string_attribute("ResolutionUnit") == "cm") {
+        // OpenEXR only supports pixels per inch, so fix the values if they
+        // came to us in cm.
+        spec.attribute ("XResolution", xdensity / 2.54f);
+        spec.attribute ("YResolution", ydensity / 2.54f);
+    }
+
+    // We must setTileDescription here before the put_parameter calls below,
+    // since put_parameter will check the header to ensure this is a tiled
+    // image before setting lineOrder to randomY.
     if (spec.tile_width)
         header.setTileDescription (
             Imf::TileDescription (spec.tile_width, spec.tile_height,
@@ -756,12 +806,16 @@ OpenEXROutput::put_parameter (const std::string &name, TypeDesc type,
         xname = "owner";
     else if (Strutil::iequals(xname, "PixelAspectRatio"))
         xname = "pixelAspectRatio";
+    else if (Strutil::iequals(xname, "XResolution"))
+        xname = "xDensity";
+    else if (Strutil::iequals(xname, "YResolution"))
+        xname = "";  // make us skip it
     else if (Strutil::iequals(xname, "ExposureTime"))
         xname = "expTime";
     else if (Strutil::iequals(xname, "FNumber"))
         xname = "aperture";
-    else if (Strutil::iequals(xname, "name"))
-        xname = "oiio::subimagename";
+    else if (Strutil::iequals(xname, "oiio:subimagename"))
+        xname = "name";
     else if (Strutil::iequals(xname, "openexr:dwaCompressionLevel"))
         xname = "dwaCompressionLevel";
 
@@ -810,7 +864,7 @@ OpenEXROutput::put_parameter (const std::string &name, TypeDesc type,
         header.lineOrder() = Imf::INCREASING_Y;   // Default
         if (str) {
             if (Strutil::iequals (str, "randomY")
-                  && m_spec.tile_width /* randomY is only for tiled files */)
+                  && header.hasTileDescription() /* randomY is only for tiled files */)
                 header.lineOrder() = Imf::RANDOM_Y;
             else if (Strutil::iequals (str, "decreasingY"))
                 header.lineOrder() = Imf::DECREASING_Y;
@@ -830,8 +884,15 @@ OpenEXROutput::put_parameter (const std::string &name, TypeDesc type,
 
     // Special handling of any remaining "oiio:*" metadata.
     if (Strutil::istarts_with (xname, "oiio:")) {
-        // None currently supported
-        return false;
+        if (Strutil::iequals (xname, "oiio:ConstantColor") ||
+            Strutil::iequals (xname, "oiio:AverageColor") ||
+            Strutil::iequals (xname, "oiio:SHA-1")) {
+            // let these fall through and get stored as metadata
+        } else {
+            // Other than the listed exceptions, suppress any other custom
+            // oiio: directives.
+            return false;
+        }
     }
 
     // Before handling general named metadata, suppress non-openexr
@@ -864,6 +925,9 @@ OpenEXROutput::put_parameter (const std::string &name, TypeDesc type,
         return false;
     }
 
+    if (! xname.length())
+        return false;    // Skip suppressed names
+
     // General handling of attributes
     try {
 
@@ -1055,6 +1119,10 @@ OpenEXROutput::put_parameter (const std::string &name, TypeDesc type,
     } catch (const std::exception &e) {
 #ifndef NDEBUG
         std::cout << "Caught OpenEXR exception: " << e.what() << "\n";
+#endif
+    } catch (...) {  // catch-all for edge cases or compiler bugs
+#ifndef NDEBUG
+        std::cout << "Caught unknown OpenEXR exception\n";
 #endif
     }
 
@@ -1142,10 +1210,12 @@ OpenEXROutput::write_scanline (int y, int z, TypeDesc format,
         } else {
             ASSERT (0);
         }
-    }
-    catch (const std::exception &e) {
+    } catch (const std::exception &e) {
         error ("Failed OpenEXR write: %s", e.what());
         return false;
+    } catch (...) {  // catch-all for edge cases or compiler bugs
+        error ("Failed OpenEXR write: unknown exception");
+        return false;
     }
 
     // FIXME -- can we checkpoint the file?
@@ -1217,10 +1287,12 @@ OpenEXROutput::write_scanlines (int ybegin, int yend, int z,
             } else {
                 ASSERT (0);
             }
-        }
-        catch (const std::exception &e) {
+        } catch (const std::exception &e) {
             error ("Failed OpenEXR write: %s", e.what());
             return false;
+        } catch (...) {  // catch-all for edge cases or compiler bugs
+            error ("Failed OpenEXR write: unknown exception");
+            return false;
         }
 
         data = (const char *)data + ystride*nscanlines;
@@ -1243,6 +1315,11 @@ OpenEXROutput::write_tile (int x, int y, int z,
                            TypeDesc format, const void *data,
                            stride_t xstride, stride_t ystride, stride_t zstride)
 {
+    bool native = (format == TypeDesc::UNKNOWN);
+    if (native && xstride == AutoStride)
+        xstride = (stride_t) m_spec.pixel_bytes (native);
+    m_spec.auto_stride (xstride, ystride, zstride, format, spec().nchannels,
+                        m_spec.tile_width, m_spec.tile_height);
     return write_tiles (x, std::min (x+m_spec.tile_width, m_spec.x+m_spec.width),
                         y, std::min (y+m_spec.tile_height, m_spec.y+m_spec.height),
                         z, std::min (z+m_spec.tile_depth, m_spec.z+m_spec.depth),
@@ -1340,10 +1417,12 @@ OpenEXROutput::write_tiles (int xbegin, int xend, int ybegin, int yend,
         } else {
             ASSERT (0);
         }
-    }
-    catch (const std::exception &e) {
+    } catch (const std::exception &e) {
         error ("Failed OpenEXR write: %s", e.what());
         return false;
+    } catch (...) {  // catch-all for edge cases or compiler bugs
+        error ("Failed OpenEXR write: unknown exception");
+        return false;
     }
 
     return true;
@@ -1392,10 +1471,12 @@ OpenEXROutput::write_deep_scanlines (int ybegin, int yend, int z,
 
         // Write the pixels
         m_deep_scanline_output_part->writePixels (yend-ybegin);
-    }
-    catch (const std::exception &e) {
+    } catch (const std::exception &e) {
         error ("Failed OpenEXR write: %s", e.what());
         return false;
+    } catch (...) {  // catch-all for edge cases or compiler bugs
+        error ("Failed OpenEXR write: unknown exception");
+        return false;
     }
 
     return true;
@@ -1459,10 +1540,12 @@ OpenEXROutput::write_deep_tiles (int xbegin, int xend, int ybegin, int yend,
         m_deep_tiled_output_part->writeTiles (firstxtile, firstxtile+xtiles-1,
                                               firstytile, firstytile+ytiles-1,
                                               m_miplevel, m_miplevel);
-    }
-    catch (const std::exception &e) {
+    } catch (const std::exception &e) {
         error ("Failed OpenEXR write: %s", e.what());
         return false;
+    } catch (...) {  // catch-all for edge cases or compiler bugs
+        error ("Failed OpenEXR write: unknown exception");
+        return false;
     }
 
     return true;
