@@ -49,8 +49,7 @@ public:
     virtual ~PNGOutput ();
     virtual const char * format_name (void) const { return "png"; }
     virtual bool supports (const std::string &feature) const {
-        // Support nothing nonstandard
-        return false;
+        return (feature == "alpha");
     }
     virtual bool open (const std::string &name, const ImageSpec &spec,
                        OpenMode mode=Create);
