@@ -1,12 +1,12 @@
 #!/usr/bin/python 
 
-# ../openexr-images-1.5.0/MultiResolution:
+# ../openexr-images/MultiResolution:
 # Bonita.exr              MirrorPattern.exr       StageEnvCube.exr
 # ColorCodedLevels.exr    OrientationCube.exr     StageEnvLatLong.exr
 # Kapaa.exr               OrientationLatLong.exr  WavyLinesCube.exr
 # KernerEnvCube.exr       PeriodicPattern.exr     WavyLinesLatLong.exr
 # KernerEnvLatLong.exr    README                  WavyLinesSphere.exr
-imagedir = parent + "/openexr-images-1.5.0/MultiResolution"
+imagedir = parent + "/openexr-images/MultiResolution"
 files = [ "Bonita.exr", "ColorCodedLevels.exr",
           # FIXME -- we don't know how to deal with RIP-maps -- Kapaa, 
           "KernerEnvCube.exr", "KernerEnvLatLong.exr", "MirrorPattern.exr",
